@@ -161,7 +161,6 @@ typedef enum
   PLUGIN_FLAG_REALIZED       = 1 << 2,
   PLUGIN_FLAG_SHOW_CONFIGURE = 1 << 3,
   PLUGIN_FLAG_SHOW_ABOUT     = 1 << 4,
-  PLUGIN_FLAG_BLOCK_AUTOHIDE = 1 << 5
 }
 PluginFlags;
 
@@ -1092,13 +1091,16 @@ xfce_panel_plugin_menu_remove (XfcePanelPlugin *plugin)
     {
       gtk_widget_hide (dialog);
 
+      /* send signal to unlock the panel before removing the plugin */
+      xfce_panel_plugin_block_autohide (plugin, FALSE);
+
       /* ask the panel or wrapper to remove the plugin */
       xfce_panel_plugin_remove (plugin);
     }
+  else
+    xfce_panel_plugin_block_autohide (plugin, FALSE);
 
   gtk_widget_destroy (dialog);
-
-  xfce_panel_plugin_block_autohide (plugin, FALSE);
 }
 
 
@@ -1407,23 +1409,14 @@ xfce_panel_plugin_unregister_menu (GtkMenu         *menu,
                                    XfcePanelPlugin *plugin)
 {
   panel_return_if_fail (XFCE_IS_PANEL_PLUGIN (plugin));
-  panel_return_if_fail (plugin->priv->panel_lock > 0);
   panel_return_if_fail (GTK_IS_MENU (menu));
 
   /* disconnect this signal */
   g_signal_handlers_disconnect_by_func (G_OBJECT (menu),
       G_CALLBACK (xfce_panel_plugin_unregister_menu), plugin);
 
-  if (G_LIKELY (plugin->priv->panel_lock > 0))
-    {
-      /* decrease the counter */
-      plugin->priv->panel_lock--;
-
-      /* emit signal to unlock the panel */
-      if (plugin->priv->panel_lock == 0)
-        xfce_panel_plugin_provider_emit_signal (XFCE_PANEL_PLUGIN_PROVIDER (plugin),
-                                                PROVIDER_SIGNAL_UNLOCK_PANEL);
-    }
+  /* tell panel it needs to unlock */
+  xfce_panel_plugin_block_autohide (plugin, FALSE);
 }
 
 
@@ -1612,8 +1605,6 @@ xfce_panel_plugin_get_show_configure (XfcePanelPluginProvider *provider)
 {
   panel_return_val_if_fail (XFCE_IS_PANEL_PLUGIN (provider), FALSE);
 
-  /* TODO, not sure, but maybe return FALSE when menu_blocked > 0 */
-
   return PANEL_HAS_FLAG (XFCE_PANEL_PLUGIN (provider)->priv->flags,
                          PLUGIN_FLAG_SHOW_CONFIGURE);
 }
@@ -1639,8 +1630,6 @@ xfce_panel_plugin_get_show_about (XfcePanelPluginProvider *provider)
 {
   panel_return_val_if_fail (XFCE_IS_PANEL_PLUGIN (provider), FALSE);
 
-  /* TODO, not sure, but maybe return FALSE when menu_blocked > 0 */
-
   return PANEL_HAS_FLAG (XFCE_PANEL_PLUGIN (provider)->priv->flags,
                          PLUGIN_FLAG_SHOW_ABOUT);
 }
@@ -1650,12 +1639,9 @@ xfce_panel_plugin_get_show_about (XfcePanelPluginProvider *provider)
 static void
 xfce_panel_plugin_show_about (XfcePanelPluginProvider *provider)
 {
-  XfcePanelPlugin *plugin = XFCE_PANEL_PLUGIN (provider);
-
   panel_return_if_fail (XFCE_IS_PANEL_PLUGIN (provider));
 
-  if (G_LIKELY (plugin->priv->menu_blocked == 0))
-    g_signal_emit (G_OBJECT (provider), plugin_signals[ABOUT], 0);
+  g_signal_emit (G_OBJECT (provider), plugin_signals[ABOUT], 0);
 }
 
 
@@ -2449,14 +2435,13 @@ xfce_panel_plugin_unblock_menu (XfcePanelPlugin *plugin)
  *
  * Register a menu that is about to popup. This will make sure the panel
  * will properly handle its autohide behaviour. You have to call this
- * function every time the menu is opened (e.g. using gtk_menu_popup()).
+ * function every time the menu is opened (e.g. using gtk_menu_popup_at_widget()).
  *
  * If you want to open the menu aligned to the side of the panel (and the
- * plugin), you should use xfce_panel_plugin_position_menu() as
- * #GtkMenuPositionFunc. This callback function will take care of calling
- * xfce_panel_plugin_register_menu() as well.
+ * plugin), you should use xfce_panel_plugin_popup_menu(). This function
+ * will take care of calling xfce_panel_plugin_register_menu() as well.
  *
- * See also: xfce_panel_plugin_position_menu() and xfce_panel_plugin_block_autohide().
+ * See also: xfce_panel_plugin_popup_menu() and xfce_panel_plugin_block_autohide().
  **/
 void
 xfce_panel_plugin_register_menu (XfcePanelPlugin *plugin,
@@ -2466,19 +2451,16 @@ xfce_panel_plugin_register_menu (XfcePanelPlugin *plugin,
   g_return_if_fail (GTK_IS_MENU (menu));
   g_return_if_fail (XFCE_PANEL_PLUGIN_CONSTRUCTED (plugin));
 
-  /* increase the counter */
-  plugin->priv->panel_lock++;
-
   /* connect signal to menu to decrease counter */
   g_signal_connect (G_OBJECT (menu), "deactivate",
       G_CALLBACK (xfce_panel_plugin_unregister_menu), plugin);
+  g_signal_connect (G_OBJECT (menu), "selection-done",
+      G_CALLBACK (xfce_panel_plugin_unregister_menu), plugin);
   g_signal_connect (G_OBJECT (menu), "destroy",
       G_CALLBACK (xfce_panel_plugin_unregister_menu), plugin);
 
   /* tell panel it needs to lock */
-  if (plugin->priv->panel_lock == 1)
-    xfce_panel_plugin_provider_emit_signal (XFCE_PANEL_PLUGIN_PROVIDER (plugin),
-                                            PROVIDER_SIGNAL_LOCK_PANEL);
+  xfce_panel_plugin_block_autohide (plugin, TRUE);
 }
 
 
@@ -2487,8 +2469,7 @@ xfce_panel_plugin_register_menu (XfcePanelPlugin *plugin,
  * xfce_panel_plugin_arrow_type:
  * @plugin : an #XfcePanelPlugin.
  *
- * Determine the #GtkArrowType for a widget that opens a menu and uses
- * xfce_panel_plugin_position_menu() to position the menu.
+ * Determine the #GtkArrowType for a widget that opens a menu.
  *
  * Returns: the #GtkArrowType to use.
  **/
@@ -2556,10 +2537,9 @@ xfce_panel_plugin_arrow_type (XfcePanelPlugin *plugin)
  * position will be relative to @plugin.
  *
  * This function is intended for custom menu widgets.
- * For a regular #GtkMenu you should use xfce_panel_plugin_position_menu()
- * instead (as callback argument to gtk_menu_popup()).
+ * For a regular #GtkMenu you should use xfce_panel_plugin_popup_menu() instead.
  *
- * See also: xfce_panel_plugin_position_menu().
+ * See also: xfce_panel_plugin_popup_menu().
  **/
 void
 xfce_panel_plugin_position_widget (XfcePanelPlugin *plugin,
@@ -2714,6 +2694,8 @@ xfce_panel_plugin_position_widget (XfcePanelPlugin *plugin,
  * xfce_panel_plugin_position_widget() instead.
  *
  * See also: gtk_menu_popup().
+ *
+ * Deprecated: 4.17.2: Use xfce_panel_plugin_popup_menu() instead.
  **/
 void
 xfce_panel_plugin_position_menu (GtkMenu  *menu,
@@ -2744,6 +2726,93 @@ xfce_panel_plugin_position_menu (GtkMenu  *menu,
 
 
 
+/**
+ * xfce_panel_plugin_popup_menu:
+ * @plugin        : an #XfcePanelPlugin.
+ * @menu          : a #GtkMenu.
+ * @widget        : (allow-none): the #GtkWidget to align @menu with or %NULL
+ *                  to pop up @menu at pointer.
+ * @trigger_event : (allow-none): the #GdkEvent that initiated this request or
+ *                  %NULL if it's the current event.
+ *
+ * Pops up @menu at @widget if @widget is non-%NULL and if appropriate given
+ * the panel position, otherwise pops up @menu at pointer.
+ *
+ * As a convenience, xfce_panel_plugin_popup_menu() calls
+ * xfce_panel_plugin_register_menu() for the @menu.
+ *
+ * For a custom widget that will be used as a popup menu, use
+ * xfce_panel_plugin_position_widget() instead.
+ *
+ * See also: gtk_menu_popup_at_widget() and gtk_menu_popup_at_pointer().
+ *
+ * Since: 4.17.2
+ **/
+void
+xfce_panel_plugin_popup_menu (XfcePanelPlugin *plugin,
+                              GtkMenu         *menu,
+                              GtkWidget       *widget,
+                              const GdkEvent  *trigger_event)
+{
+  GdkGravity widget_anchor, menu_anchor;
+  gboolean   popup_at_widget = TRUE;
+
+  g_return_if_fail (XFCE_IS_PANEL_PLUGIN (plugin));
+  g_return_if_fail (GTK_IS_MENU (menu));
+
+  /* check if conditions are met to pop up menu at widget */
+  if (widget != NULL)
+    {
+      switch (plugin->priv->screen_position)
+        {
+          case XFCE_SCREEN_POSITION_NW_H:
+          case XFCE_SCREEN_POSITION_N:
+          case XFCE_SCREEN_POSITION_NE_H:
+            widget_anchor = GDK_GRAVITY_SOUTH_WEST;
+            menu_anchor = GDK_GRAVITY_NORTH_WEST;
+            break;
+
+          case XFCE_SCREEN_POSITION_NW_V:
+          case XFCE_SCREEN_POSITION_W:
+          case XFCE_SCREEN_POSITION_SW_V:
+            widget_anchor = GDK_GRAVITY_NORTH_EAST;
+            menu_anchor = GDK_GRAVITY_NORTH_WEST;
+            break;
+
+          case XFCE_SCREEN_POSITION_NE_V:
+          case XFCE_SCREEN_POSITION_E:
+          case XFCE_SCREEN_POSITION_SE_V:
+            widget_anchor = GDK_GRAVITY_NORTH_WEST;
+            menu_anchor = GDK_GRAVITY_NORTH_EAST;
+            break;
+
+          case XFCE_SCREEN_POSITION_SW_H:
+          case XFCE_SCREEN_POSITION_S:
+          case XFCE_SCREEN_POSITION_SE_H:
+            widget_anchor = GDK_GRAVITY_NORTH_WEST;
+            menu_anchor = GDK_GRAVITY_SOUTH_WEST;
+            break;
+
+          default:
+            popup_at_widget = FALSE;
+            break;
+        }
+    }
+  else
+    popup_at_widget = FALSE;
+
+  /* register the menu */
+  xfce_panel_plugin_register_menu (plugin, menu);
+
+  /* pop up the menu */
+  if (popup_at_widget)
+    gtk_menu_popup_at_widget (menu, widget, widget_anchor, menu_anchor, trigger_event);
+  else
+    gtk_menu_popup_at_pointer (menu, trigger_event);
+}
+
+
+
 /**
  * xfce_panel_plugin_focus_widget:
  * @plugin : an #XfcePanelPlugin.
@@ -2780,8 +2849,9 @@ xfce_panel_plugin_focus_widget (XfcePanelPlugin *plugin,
  * plugin at it will look weird for a user if the panel will hide while
  * he/she is working in the popup.
  *
- * For menus use xfce_panel_plugin_register_menu() which will take care
- * of this.
+ * Be sure to use this function as lock/unlock pairs, as a counter is
+ * incremented/decremented under the hood. For menus, you can use
+ * xfce_panel_plugin_register_menu() which will take care of this.
  **/
 void
 xfce_panel_plugin_block_autohide (XfcePanelPlugin *plugin,
@@ -2790,18 +2860,11 @@ xfce_panel_plugin_block_autohide (XfcePanelPlugin *plugin,
   g_return_if_fail (XFCE_IS_PANEL_PLUGIN (plugin));
   g_return_if_fail (XFCE_PANEL_PLUGIN_CONSTRUCTED (plugin));
 
-  /* leave when requesting the same block state */
-  if (PANEL_HAS_FLAG (plugin->priv->flags, PLUGIN_FLAG_BLOCK_AUTOHIDE) == blocked)
-    return;
-
   if (blocked)
     {
       /* increase the counter */
-      panel_return_if_fail (plugin->priv->panel_lock >= 0);
       plugin->priv->panel_lock++;
 
-      PANEL_SET_FLAG (plugin->priv->flags, PLUGIN_FLAG_BLOCK_AUTOHIDE);
-
       /* tell panel it needs to lock */
       if (plugin->priv->panel_lock == 1)
         xfce_panel_plugin_provider_emit_signal (XFCE_PANEL_PLUGIN_PROVIDER (plugin),
@@ -2813,8 +2876,6 @@ xfce_panel_plugin_block_autohide (XfcePanelPlugin *plugin,
       panel_return_if_fail (plugin->priv->panel_lock > 0);
       plugin->priv->panel_lock--;
 
-      PANEL_UNSET_FLAG (plugin->priv->flags, PLUGIN_FLAG_BLOCK_AUTOHIDE);
-
       /* tell panel it needs to unlock */
       if (plugin->priv->panel_lock == 0)
         xfce_panel_plugin_provider_emit_signal (XFCE_PANEL_PLUGIN_PROVIDER (plugin),
