@@ -51,7 +51,7 @@
 
 /* Use the posix directory for the names. If people want a time based on posix or
  * right time, they can prepend that manually in the entry */
-#define ZONEINFO_DIR "/usr/share/zoneinfo/"
+#define ZONEINFO_DIR "/usr/share/zoneinfo/posix/"
 
 
 
@@ -971,9 +971,6 @@ clock_plugin_configure_zoneinfo_model_insert (GtkListStore *store,
 
   panel_return_if_fail (GTK_IS_LIST_STORE (store));
 
-  if (g_str_has_suffix (parent, "posix") || g_str_has_suffix (parent, "right"))
-    return;
-
   dir = g_dir_open (parent, 0, NULL);
   if (dir == NULL)
     return;
@@ -1206,8 +1203,8 @@ clock_plugin_reposition_calendar (ClockPlugin *plugin)
 
 
 static void
-clock_plugin_calendar_show_event (GtkWidget   *calendar_window,
-                                  ClockPlugin *plugin)
+clock_plugin_calendar_realized (GtkWidget   *calendar_window,
+                                ClockPlugin *plugin)
 {
   GDateTime *time;
 
@@ -1371,8 +1368,8 @@ clock_plugin_popup_calendar (ClockPlugin *plugin)
                                         GTK_CALENDAR_SHOW_HEADING
                                         | GTK_CALENDAR_SHOW_DAY_NAMES
                                         | GTK_CALENDAR_SHOW_WEEK_NUMBERS);
-      g_signal_connect (G_OBJECT (plugin->calendar_window), "show",
-                        G_CALLBACK (clock_plugin_calendar_show_event), plugin);
+      g_signal_connect (G_OBJECT (plugin->calendar_window), "realize",
+                        G_CALLBACK (clock_plugin_calendar_realized), plugin);
       g_signal_connect (G_OBJECT (plugin->calendar_window), "button-press-event",
                         G_CALLBACK (clock_plugin_calendar_button_press_event), plugin);
       g_signal_connect (G_OBJECT (plugin->calendar_window), "key-press-event",
