@@ -433,7 +433,7 @@ sn_item_properties_callback (GObject      *source_object,
 
   g_signal_connect (item->item_proxy, "g-signal",
                     G_CALLBACK (sn_item_signal_received), item);
-  sn_item_invalidate (item);
+  sn_item_invalidate (item, FALSE);
 }
 
 
@@ -519,7 +519,8 @@ sn_item_start (SnItem *item)
 
 
 void
-sn_item_invalidate (SnItem *item)
+sn_item_invalidate (SnItem   *item,
+                    gboolean  force_update)
 {
   g_return_if_fail (XFCE_IS_SN_ITEM (item));
 
@@ -527,6 +528,16 @@ sn_item_invalidate (SnItem *item)
   if (item->properties_proxy == NULL)
     return;
 
+  if (force_update)
+    {
+      /* force menu update (see issue #567) */
+      if (item->menu_object_path != NULL)
+        {
+          g_free (item->menu_object_path);
+          item->menu_object_path = NULL;
+        }
+    }
+
   g_dbus_proxy_call (item->properties_proxy,
                      "GetAll",
                      g_variant_new ("(s)", "org.kde.StatusNotifierItem"),
@@ -564,7 +575,7 @@ sn_item_signal_received (GDBusProxy *proxy,
       !g_strcmp0 (signal_name, "NewOverlayIcon") ||
       !g_strcmp0 (signal_name, "NewToolTip"))
     {
-      sn_item_invalidate (item);
+      sn_item_invalidate (item, FALSE);
     }
   else if (!g_strcmp0 (signal_name, "NewStatus"))
     {
